#!/usr/bin/python3
import sys
import configparser
import sqlite3
import os
import apt
import apt_pkg
import shutil

# 创建自定义解析器类
class CaseInsensitiveConfigParser(configparser.ConfigParser):
    def __init__(self, *args, **kwargs):
        super().__init__(*args, **kwargs)
        # 重写 optionxform 方法，将所有选项名转为小写
        self.optionxform = lambda option: option.lower()

def merge_ini_files(file1, file2):
    """
    将file1中有而file2中没有的配置项写入file2
    (配置项不区分大小写)
    
    参数:
        file1 (str): 源INI文件路径
        file2 (str): 目标INI文件路径，将被更新
    
    返回:
        None
    """
    # 创建临时文件用于备份
    backup_file = file2 + ".bak"
    
    try:
        # 备份原始文件
        shutil.copy2(file2, backup_file)
        
        # 创建不区分大小写的parser
        class CaseInsensitiveConfigParser(configparser.ConfigParser):
            def __init__(self, *args, **kwargs):
                super().__init__(*args, **kwargs)
                self.optionxform = lambda option: option.lower()
                
            def get_section(self, section_name):
                """获取不区分大小写的section名称"""
                section_name_lower = section_name.lower()
                for sec in self.sections():
                    if sec.lower() == section_name_lower:
                        return sec
                return None
            
            def get_option(self, section, option):
                """获取不区分大小写的option名称"""
                section_name = self.get_section(section)
                if not section_name:
                    return None
                    
                option_lower = option.lower()
                for opt in self.options(section_name):
                    if opt.lower() == option_lower:
                        return opt
                return None
        
        # 读取文件
        parser1 = CaseInsensitiveConfigParser()
        parser2 = CaseInsensitiveConfigParser()
        
        # 保留原始大小写
        parser1.optionxform = lambda option: option
        parser2.optionxform = lambda option: option
        
        parser1.read(file1)
        parser2.read(file2)
        
        # 遍历file1的所有section
        for section1 in parser1.sections():
            section_lower = section1.lower()
            
            # 在file2中查找对应section(不区分大小写)
            section2 = parser2.get_section(section1)
            
            # 如果file2中没有这个section，添加整个section
            if not section2:
                parser2.add_section(section1)
                section2 = section1
            
            # 遍历file1该section的所有选项
            for option1 in parser1.options(section1):
                option_lower = option1.lower()
                
                # 检查file2中是否存在(不区分大小写)
                option2 = parser2.get_option(section2, option1)
                
                # 如果选项不存在，添加原始大小写的选项和值
                if not option2:
                    parser2.set(section2, option1, parser1.get(section1, option1))
        
        # 写回更新后的内容到file2
        with open(file2, 'w') as f:
            parser2.write(f)
        
        print(f"文件已更新: {file2}")
        print(f"原始文件已备份为: {backup_file}")
        
    except Exception as e:
        # 恢复备份文件
        if 'backup_file' in locals() and shutil.os.path.exists(backup_file):
            shutil.copy2(backup_file, file2)
            print(f"操作出错，已恢复备份: {e}")
        else:
            print(f"操作出错: {e}")
            raise

if __name__ == "__main__":
    new_config_file=sys.argv[1]
    old_config_file=sys.argv[2]
    db_file="/var/lib/kylin-system-updater/kylin-system-updater.db" 
    # new_config_file = "/usr/share/kylin-system-updater/unattended-upgrades-policy.conf"
    # old_config_file = "/var/lib/unattended-upgrades/unattended-upgrades-policy.conf"
    # config_new=configparser.ConfigParser(allow_no_value=True)
    # config_new.optionxform = str
    # config_old=configparser.ConfigParser(allow_no_value=True)
    # config_old.optionxform = str
    # config_new.read(new_config_file)
    # config_old.read(old_config_file)
    # try:
    #     for section in config_new.sections():
    #         if not config_old.has_section(section):
    #             config_old.add_section(section)
    #             print("new section %s added"%(section))  
    #         for option in config_new.options(section):
    #             if not config_old.has_option(section,option):
    #                 value = config_new[section][option]
    #                 config_old.set(section,option,value)
    #                 print("new option:%s %s"%(option,value))
    #     with open(old_config_file,'w') as f:
    #         config_old.write(f)           
    # except Exception as e:
    #     print(e)
    print("merge %s and %s"%(new_config_file,old_config_file))
    merge_ini_files(new_config_file,old_config_file)
    dbrefreshflag="/var/lib/db-refresh-flag"
    if(os.path.exists(dbrefreshflag)):
        print("%s already refreshed,pass"%old_config_file)
    else:
        print("refresh %s with old configs"%old_config_file)
        cache = apt.Cache()
        if('ukui-control-center' in cache):
            ukcc=cache['ukui-control-center']
            ukcc_version=ukcc.installed.version
            print(ukcc_version)
            cmpret=apt_pkg.version_compare(ukcc_version,'3.1.1+2022-0324')
        autoupdate_allow="false"
        if (cmpret>=0):
            print("system version greater than or equal to 2203")
            if (os.path.exists(db_file)):
                conn = sqlite3.connect(db_file)
                cursor = conn.cursor()
                cursor.execute('SELECT autoupdate_allow FROM display')
                dat = cursor.fetchone()
                autoupdate_allow=dat[0]
                print("auto update allow in db:%s"%autoupdate_allow)
        else:
            print("ukcc version lower than 2203")    
            configfile_2107 = "/var/lib/kylin-auto-upgrade/kylin-autoupgrade.conf"
            if (os.path.exists(configfile_2107)):
                config_2107=CaseInsensitiveConfigParser()
                config_2107.optionxform = str
                config_2107.read(configfile_2107)
                autoupdate_allow=config_2107['CONTROL_CENTER']['autoupdate_allow']
        config_old = CaseInsensitiveConfigParser()
        config_old.read(old_config_file)
        if(autoupdate_allow=="true"):
            config_old.set('autoUpgradePolicy','autoUpgradeState','on')
        elif(autoupdate_allow=="false"):
            config_old.set('autoUpgradePolicy','autoUpgradeState','off')
        with open(old_config_file,'w') as f:
            config_old.write(f)  
        with open(dbrefreshflag,'w') as f:
                f.write("ini already refreshed")
