import os
import time
from gettext import gettext as _
import logging
import subprocess
from gi.repository import GLib
from SystemUpdater.Core.system import run_system

CHECK_IDLE_INTERVAL = 5

class PlymouthManager():
    def __init__(self):
        self.plymouth_timestamp         = 0
        self.plymouth_check_interval    = -1

    def message_to(self, message):
        subprocess.call(["/bin/plymouth", "message", "--text", message])

    def progress_to(self, progress):
        tmp = ("--progress=%d"%progress)
        run_cmd = ["/bin/plymouth", "system-update", tmp]
        subprocess.call(run_cmd)

    def show(self):
        try:
            self._plymouth_log_rotate()
            self._plymouth_show_splash()

            self.plymouth_check_interval = 10
            GLib.timeout_add_seconds (CHECK_IDLE_INTERVAL,
                                      self._plymouth_inactivity)
        except Exception as exc:
            logging.error(exc)

    def show_countdown(self,success):
        if success:
            msg = _("The system update is successful and your computer will restart in %s s")
        else:
            msg = _("System update is abnormal and your computer will restart in %s s")

        for i in range(5, 0, -1):
            msges = msg%str(i)
            self.message_to(msges)
            time.sleep(1)
            logging.info(msges)

    def quit_retain(self):
        if os.path.exists("/bin/plymouth"):
            subprocess.run(["/bin/plymouth","quit","--retain-splash"])

    def clean_all(self):
        self.message_to("")
        self.progress_to(0)


    def _check_device_type(self):
        try:
            with open("/proc/cpuinfo", 'r') as f:
                for line in f.readlines():
                    if "Hardware" in line and ":" in line:
                        cpuinfos = line.split(':')
                        if "PANGU M900" in cpuinfos[1]:
                            logging.info("Get cpuinfo: PANGU M900")
                            return True
        except Exception as exc:
            logging.error(exc)
        return False

    def _check_plymouth_status(self):
        try:
            args = ["/bin/plymouth", "--ping"]
            p = subprocess.run(args, timeout=5, stdout=subprocess.PIPE, stderr=subprocess.STDOUT, text=True)
            if p.returncode == 0:
                args = ["/bin/plymouth", "--has-active-vt"]
                p = subprocess.run(args, timeout=5, stdout=subprocess.PIPE, stderr=subprocess.STDOUT, text=True)
                if p.returncode == 0:
                    logging.info("Check: Plymouthd is running and has active vt ...")
                else:
                    logging.info("Check: Plymouthd has no active vt, force switch tty ...")
                    self._switch_to_tty()
            else:
                logging.info("Check: Plymouthd is not running, try to start ...")
                self._plymouth_show_splash()
        except Exception as e:
            logging.error("_check_plymouth_status error: %s", e)

    def _switch_to_tty(self):
        run_system(["/usr/bin/chvt", "63"])

    def _plymouth_show_splash(self):
        if os.path.exists("/sbin/plymouthd"):
            if self._check_device_type():
                os.environ["DISPLAY"] = ":0"

            args = [
                "/sbin/plymouthd",
                "--mode=shutdown",
                "--attach-to-session",
                "--kernel-command-line=splash plymouth.debug=stream:/var/log/kylin-system-updater/plymouth-system-updater.log"
            ]

            logging.info("Running plymouth:%r", args)
            subprocess.run(args, timeout=5)

        if os.path.exists("/bin/plymouth"):
            subprocess.Popen(["/bin/plymouth", "show-splash", "--wait"])

    def _plymouth_log_rotate(self):
        try:
            subprocess.run(["logrotate", "-f", "/etc/logrotate.d/plymouth-system-updater"])
        except Exception as e:
            logging.error("plymouth_log_rotate error: %s", e)

    def _plymouth_inactivity(self):
        timestamp = self.plymouth_timestamp
        if timestamp == 0:
            self.plymouth_timestamp = time.time()
            return True

        if self.plymouth_timestamp != 0 and  time.time() - self.plymouth_timestamp > self.plymouth_check_interval:
            if self.plymouth_check_interval == 10:
                self.plymouth_check_interval = 60
            elif self.plymouth_check_interval == 60:
                self.plymouth_check_interval = 5 * 60
            elif self.plymouth_check_interval == 5 * 60:
                self.plymouth_check_interval = -1
            else:
                logging.info("_plymouth_inactivity timeout check quit ...")
                return False

            self._check_plymouth_status()

        return True
