#!/usr/bin/python3
from configparser import NoOptionError
from configparser import ConfigParser as SafeConfigParser
import os.path
import os
import shutil
import logging

def validate_ini_file(file_path: str) -> bool:
    """
    验证文件是否为合法INI格式
    
    参数:
    file_path - 待检测文件路径
    
    返回:
    bool - 可被configparser正常解析返回True，否则返回False
    
    功能特性:
    - 自动处理文件编码问题
    - 支持所有标准INI语法结构验证
    - 覆盖常见错误类型检测
    """
    parser = SafeConfigParser()
    try:
        # 尝试读取文件并检测有效内容
        read_files = parser.read(file_path, encoding='utf-8-sig')  # 处理BOM头
        
        # 文件不存在或无法解码时返回False
        if not read_files:
            return False
        
        # 验证至少存在一个有效节或键值对
        return len(parser.sections()) > 0 or len(parser.defaults()) > 0
    except Exception as e:  # 兜底处理其他异常
        print(f"Unexpected error: {str(e)}")
        return False

def check_ini_files(source,dest):
    if(validate_ini_file(dest)):
        pass
    elif(validate_ini_file(source)):
        logging.warning("file %s illegal ini format,copy from %s"%(dest,source))
        shutil.copyfile(source,dest)

class UpgradeConfig(SafeConfigParser):
    # 备份还原之前保存的配置
    UPDATE_NOT_RESOTRE_DIR = "/var/lib/kylin-system-updater/remain"

    def __init__(self, datadir="/var/lib/kylin-system-updater/", name="system-updater.conf",defaults_conf=None):
        SafeConfigParser.__init__(self)
        check_ini_files("/usr/share/kylin-system-updater/unattended-upgrades-policy.conf","/var/lib/unattended-upgrades/unattended-upgrades-policy.conf")
        self.datadir = datadir
        self.maincfg = os.path.join(datadir, name)

        self.config_files = []
        # 增加的默认配置，先读取默认配置加载配置项，最终写入到主配置内
        if defaults_conf:
            self.config_files.append(os.path.join(datadir, defaults_conf))
        self.config_files.append(self.maincfg)
        
        # 检查是否存在备份的配置，存在时直接移动到默认位置
        self.remail_conf = os.path.join(self.UPDATE_NOT_RESOTRE_DIR, name)
        if os.path.exists(self.remail_conf):
            shutil.move(self.remail_conf,self.maincfg)

        # 检查配置文件是否存在
        for conf in self.config_files:
            if not os.path.exists(conf):
                logging.warning("Check: Conf(%s) not exits...",conf)
                # self.config_files.remove(conf)

        self.read(self.config_files, encoding='iso-8859-1')
        logging.info("Initialize Upgrade ConfigFile(%s) to success",str(self.config_files))

    def reRead(self):
        self.read(self.config_files, encoding='iso-8859-1')

    def setValue(self, section, option, value=None,is_write = True):
        logging.info("SetValue Section:%s Option:%s Value:%s",section, option, value)
        try:
            self.reRead()
            self.set(section, option,value)
        except Exception as e:
                logging.error("Error: setValue section:%s option:%s value:%s",section, option, value)
                logging.error(str(e))
                return False
        if is_write == True:
            with open(self.maincfg, 'w+') as configfile:
                self.write(configfile)
        return True

    def getWithDefault(self, section, option, default,re_read=False):
        try:
            if re_read == True:
                self.reRead()

            if type(default) == bool:
                return self.getboolean(section, option)
            elif type(default) == float:
                return self.getfloat(section, option)
            elif type(default) == int:
                return self.getint(section, option)
            return self.get(section, option)
        except Exception as e:
            logging.warning(str(e))
            return default

    def getlist(self, section, option):
        try:
            tmp = self.get(section, option)
        except Exception as e:
            logging.error(str(e))
            return []
        items = [x.strip() for x in tmp.split(" ")]
        if '' in items and len(items):
            return []
        return items

    def setListValue(self, section, option, value=None,is_write = True):
        tmp = str(value).replace('[', '').replace(']', '')
        tmp = tmp.replace("'", '').replace(',', '')
        try:
            self.set(section, option,tmp)
        except Exception as e:
            logging.error("setListValue section:%s option:%s",section, option)
            logging.error(str(e))
            return
        if is_write == True:
            with open(self.maincfg, 'w+') as configfile:
                self.write(configfile)

    def getListFromFile(self, section, option):
        try:
            filename = self.get(section, option)
        except NoOptionError:
            return []
        p = os.path.join(self.datadir, filename)
        if not os.path.exists(p):
            logging.error("getListFromFile: no '%s' found" % p)
        with open(p) as f:
            items = [x.strip() for x in f]
        return [s for s in items if not s.startswith("#") and not s == ""]

    # 还原之前保留配置
    def remain_conf(self):
        shutil.copy(self.maincfg, self.remail_conf)

    def optionxform(self, optionstr: str) -> str:
        return optionstr
if __name__ == "__main__":
    pass